;=========================================================
;
; Variable Length Codes encoded smooth scroller
; for c64 and/or c128!
; (c) 2010, 2018 ccr/TNSP^pWp <ccr@tnsp.org>
; assistance on C128 matters provided by XmikeX
;
;=========================================================

    scrrow      = 15
    screen      = $0400
    scroll      = screen + (40 * scrrow)
    colors      = $d800 + (40 * scrrow)
    data_len    = data_end - data

;=========================================================


#ifdef C128
    ; C128 build
    orig = $0b00    ; C128 bootsector buffer

    * = orig
    .byte "CBM"     ; Boot signature
    .word $0c00     ; Load address for additional sectors
    .byte $f        ; RAM bank config for additonal sectors
    .byte 1         ; how many sectors to load

    .byte $93       ; clear screen byte
    .byte 0         ; end of boot text data

#else
    ; C64 build
    orig = $0326    ; load at basic's CHROUT vector
    .word orig
    * = orig
    .word start     ; set CHROUT vector to our start address
    .word $f6ed     ; appease the cruel gods here
#endif

;=========================================================
code1:  jsr getmbits
    
setchr: tax
    lda codetab,x
    sta scroll+39

;=========================================================

start:  ; Initialize, screen colours
    sei
    ldy #0      ; we need Y = 0 later

    ; Main loop
main:
#ifdef C128
    ; C128 version waits for raster few times
    ldx #2      ; scroll slowdown
raswt2:
    ldy #$40
raswt3:
    nop
    dey
    bne raswt3 

    lda #$bf    ; wait for 
raswt1: cmp $d012
    bne raswt1

    dex
    bne raswt2 
#else
    ; C64 version
    jsr $ba78   ; this idea was stolen from TNT/BF :P

    lda #$bf
raswt1: cmp $d012
    bne raswt1
#endif

enext:
    .byte $cf   ; dcp $d016
    .word $d016
    bne main

    ; Move screen, fetch moar data
    ; Y = 0
movelp: lda scroll+1,y
    sta scroll,y
    lda #7      ; make it colourful
    sta colors,y
    iny
    bne movelp
    ; Y = 0 again

    sta $d016   ; ... :P
    asl

#ifdef C128
    sta $d020
#endif
    sta $d021

;=========================================================
; Decompress variable length code encoded text
;=========================================================

    jsr getmbits
    beq main    ; trust that the scrolled in char is space ...

    lsr
    beq code1
    bcc code2

code3:
    jsr getmbits
    jsr get2bits
    adc #12
    bne setchr

code2:
    jsr getmbits
    jsr getbit
    adc #4
    bne setchr

;=========================================================
; Fetch 2 bits from datastream, return in A
; also changes flags and X register
;=========================================================
getmbits:
    tya ; assume Y = 0
get2bits:
    jsr getbit

;=========================================================
; Fetch one bit from the datastream, return it in
; A register, changes X register.
; This is done in size-optimized, really wasteful way
; by rol'ing the whole compressed data section to get the
; bits in order ... This also limits max data length to
; 255 bytes, of course.
;=========================================================
getbit:
    ldx #data_len
bitlp:  rol data, x
    dex
    bne bitlp
    rol
    rts

;=========================================================
; Decoding table
#ifdef C128
#include "text_128.txt.asm"
#else
#include "text_64.txt.asm"
#endif

; Compressed scroller text data 
data:
    .byte 0
#ifdef C128
    .bin 0,0,"text_128.dat"
#else
    .bin 0,0,"text_64.dat"
#endif
data_end:
